<?php
// phpcs:disable WordPress.Security.NonceVerification -- disable nonce warning

use Automattic\Jetpack\Sync\Functions;

if (!defined('ABSPATH')) {
	exit; // Exit if accessed directly.
}

/**
 * payOS Transfer Payment Gateway.
 *
 * Provides a payOS Payment Gateway. Based on code by Mike Pepper.
 *
 * @class       WC_Gateway_CASSO_payOS
 * @extends     WC_Payment_Gateway
 * @version     2.1.0
 * @package     WooCommerce\Classes\Payment
 */
class WC_Gateway_CASSO_payOS extends WC_Payment_Gateway
{

	/**
	 * Array of locales
	 *
	 * @var array
	 */
	public $locale;

	/**
	 * PayOS gateway settings
	 * 
	 * @var array 
	 */
	public $payos_gateway_settings;

	public $message;

	public $developer_id;

	public $account_details;

	public $instructions;

	/**
	 * Constructor for the gateway.
	 */
	static private $URL_CREATE_PAYMENT_LINK = "https://api-merchant.payos.vn/v2/payment-requests";
	static private $URL_CONFIRM_WEBHOOK = "https://api-merchant.payos.vn/confirm-webhook";
	static private $URL_GET_PAYMENT_LINK_INFO = "https://api-merchant.payos.vn/v2/payment-requests/";
	static private $PAYOS_SUCCESS = "00";
	static private $PAYOS_NOT_FOUND_ORDER_CODE = 101;
	static private $PAYOS_CHECKOUT_HOST = "https://pay.payos.vn";
	static private $WEBHOOK_ROUTE = "verify_payos_webhook";
	static private $default_settings = array(
		'use_payment_gateway'         => 'yes',
		'client_id' => '',
		'api_key' => '',
		'checksum_key' => '',
		'order_status' => array(
			'order_status_after_paid'   => 'wc-completed',
			'order_status_after_underpaid' => 'wc-processing',
		),
		'transaction_prefix' => 'DH',
		'redirect' => 'yes',
		'link_webhook' => 'no',
		'gateway_info' => array(
			'name' => '',
			'account_number' => '',
			'account_name' => '',
			'bank_name' => ''
		)
	);
	static private $BLACK_LIST_PREFIX = array("FT", "TF", "TT", "VQR");
	public function __construct()
	{
		$this->id                 = 'casso-payos';
		$this->icon               = apply_filters('woocommerce_payos_icon', '');
		$this->has_fields         = false;
		$this->icon               = apply_filters('woocommerce_icon_payos', plugins_url('../assets/img/payos_crop.png', __FILE__));
		$this->method_title       = __('Payment by bank transfer (Scan VietQR)', 'casso-payos');
		$this->method_description = __('Take payments by scanning QR code with Vietnamese banking App. Supported by most major banks in Vietnam', 'casso-payos');

		global $wp_session;

		$this->message = '';

		// Load the settings.
		$this->init_form_fields();
		$this->init_settings();

		// Define user set variables.
		$this->title        = $this->get_option('title');
		$this->description  = $this->get_option('description');

		//TODO: xóa dòng dưới khi đã sang năm 2023
		if (strpos($this->description, "Hơn 14 ngân hàng Việt Nam") !== false) {
			$this->description = str_replace("Hơn 14 ngân hàng Việt Nam", "Hầu hết ngân hàng Việt Nam", $this->description);
			$this->update_option("description", $this->description);
		}

		$this->developer_id = $this->get_option('developer_id');
		// payOS account fields shown on the thanks page and in emails.

		$this->payos_gateway_settings = self::get_payos_gateway_settings();

		if (isset($_REQUEST['payos_gateway_settings']) && isset($_REQUEST['submit'])) {
			$this->save_settings_and_webhook();
		}

		add_action('woocommerce_update_options_payment_gateways_' . $this->id, array($this, 'process_admin_options'));
		add_action('woocommerce_update_options_payment_gateways_' . $this->id, array($this, 'save_settings_and_reset_webhook'));
		add_action('woocommerce_thankyou_' . $this->id, array($this, 'thankyou_page'), 5);
		// Customer Emails.
		add_action('woocommerce_email_before_order_table', array($this, 'email_instructions'), 10, 3);
		add_action('woocommerce_api_payos/developer', array($this, 'payos_developer_payment_handler'));
		// Customize Order Button
		add_action('woocommerce_after_checkout_validation', array($this, 'check_payos_gateway_settings'), 10, 2);
		// Code for add tax_code field
		// add_action('woocommerce_after_checkout_billing_form', array($this, 'custom_tax_code_field'));
		// add_action('woocommerce_checkout_update_order_meta', array($this, 'update_tax_code_to_order_meta'));

		add_action('woocommerce_api_' . self::$WEBHOOK_ROUTE, array($this, 'verify_payment_handler'));
		add_action('admin_notices', array($this, 'payos_notice_checkout'));

		if (wc_get_checkout_url() == 'https://mitom.devgioi.com/checkout/' || wc_get_checkout_url() == 'http://woo.dev.com/checkout/') {
			add_filter('woocommerce_checkout_fields', array($this, 'remove_checkout_fields'));
		}

		// custom thankyou text
		add_filter('woocommerce_thankyou_order_received_text', array($this, 'order_received_text'), 10, 2);
	}

	public function remove_checkout_fields($fields)
	{
		unset($fields['billing']['billing_company']);
		unset($fields['billing']['billing_address_2']);
		unset($fields['billing']['billing_postcode']);
		unset($fields['order']['order_comments']);
		$fields['billing']['billing_city']['default'] = 'HCM';
		$fields['billing']['billing_address_1']['default'] = 'Khu CNPM ĐHQG, TP.Thu Duc, TP.HCM';
		return $fields;
	}

	public function payos_notice_checkout()
	{
		$payos_gateway_settings = self::get_payos_gateway_settings();
		if (
			!isset($payos_gateway_settings['client_id']) || strlen($payos_gateway_settings['client_id']) == 0
			|| !isset($payos_gateway_settings['api_key']) || strlen($payos_gateway_settings['api_key']) == 0
			|| !isset($payos_gateway_settings['checksum_key']) || strlen($payos_gateway_settings['checksum_key']) == 0
		) {
			echo '<div class="notice notice-warning">
					<p>' . esc_html__('payOS has not been set up yet!', 'casso-payos') . '</p>
				</div>';
		}
	}

	public function check_payos_gateway_settings($data, $error)
	{
		if ($data['payment_method'] === $this->id) {
			$payos_gateway_settings = self::get_payos_gateway_settings();
			if (
				!isset($payos_gateway_settings['client_id']) || strlen($payos_gateway_settings['client_id']) == 0
				|| !isset($payos_gateway_settings['api_key']) || strlen($payos_gateway_settings['api_key']) == 0
				|| !isset($payos_gateway_settings['checksum_key']) || strlen($payos_gateway_settings['checksum_key']) == 0
			) {
				wc_add_notice(__('payOS has not been set up by the administrator. Please contact the system administrator.', 'casso-payos'), 'error');
			}
		}
	}

	public function save_settings_and_webhook()
	{
		if (is_array($_REQUEST['payos_gateway_settings'])) {
			$create_webhook = self::payos_create_webhook($_REQUEST['payos_gateway_settings']);
			if ($create_webhook) {
				$gateway_info = json_decode($create_webhook, true);
				$this->update_payos_gateway_settings("yes", $gateway_info, $_REQUEST['payos_gateway_settings']);

				$this->message = '<div class="updated notice"><p><strong>' .
					__('Successful webhook registration', 'casso-payos') .
					'</p></strong></div>';
			} else {
				$this->update_payos_gateway_settings("no", self::$default_settings['gateway_info']);

				$this->message =
					'<div class="error notice"><p><strong>' .
					__('Webhook creation failed', 'casso-payos') .
					'</p></strong></div>';
			}
			// Message for use
			$this->message .=
				'<div class="updated notice"><p><strong>' .
				__('Settings saved', 'casso-payos') .
				'</p></strong></div>';
		} else {
			$this->update_payos_gateway_settings("no", self::$default_settings['gateway_info']);
			$this->message =
				'<div class="error notice"><p><strong>' .
				__('Can not save settings! Please refresh this page.', 'casso-payos') .
				'</p></strong></div>';
		}
	}

	public function custom_tax_code_field($checkout)
	{
		woocommerce_form_field('tax_code', array(
			'type'          => 'text',
			'class'         => array('my-field-class form-row-wide'),
			'label'         => __('Mã số thuế'),
			'placeholder'   => __('Nhập mã số thuế'),
		), $checkout->get_value('tax_code'));
	}

	public function update_tax_code_to_order_meta($order_id)
	{
		$order = wc_get_order($order_id);
		if (!empty($_POST['tax_code'])) {
			$order->update_meta_data('tax_code', sanitize_text_field($_POST['tax_code']));
			$order->save();
		}
	}

	public function update_payos_gateway_settings($webhook_status, $gateway_info, $gateway_settings = null)
	{
		if ($webhook_status == 'no') {
			$this->payos_gateway_settings['gateway_info'] = $gateway_info;
		} else {
			$this->payos_gateway_settings = array_merge($this->payos_gateway_settings, $gateway_settings);
			$this->payos_gateway_settings['gateway_info']['account_name'] = $gateway_info['data']['accountName'];
			$this->payos_gateway_settings['gateway_info']['account_number'] = $gateway_info['data']['accountNumber'];
			$this->payos_gateway_settings['gateway_info']['name'] = $gateway_info['data']['name'];
			$this->payos_gateway_settings['gateway_info']['bank_name'] = $gateway_info['data']['shortName'];
		}
		$this->payos_gateway_settings['link_webhook'] = $webhook_status;
		update_option("woocommerce_payos_gateway_settings", $this->payos_gateway_settings);
	}

	static function get_payos_gateway_settings()
	{
		$payos_settings = get_option('woocommerce_payos_gateway_settings', self::$default_settings);
		$payos_settings = wp_parse_args($payos_settings, self::$default_settings);
		return $payos_settings;
	}

	public function save_settings_and_reset_webhook()
	{
		if (is_array($_REQUEST['payos_gateway_settings'])) {
			if (
				$_REQUEST['payos_gateway_settings']['client_id'] != $this->payos_gateway_settings['client_id']
				|| $_REQUEST['payos_gateway_settings']['api_key'] != $this->payos_gateway_settings['api_key']
				|| $_REQUEST['payos_gateway_settings']['checksum_key'] != $this->payos_gateway_settings['checksum_key']
			) {
				$_REQUEST['payos_gateway_settings']['link_webhook'] = 'no';
			}

			$_REQUEST['payos_gateway_settings']['transaction_prefix'] = preg_replace('/[^a-zA-Z0-9]/', '', $_REQUEST['payos_gateway_settings']['transaction_prefix']);

			if (strlen($_REQUEST['payos_gateway_settings']['transaction_prefix']) > 3) {
				$_REQUEST['payos_gateway_settings']['transaction_prefix'] = substr($_REQUEST['payos_gateway_settings']['transaction_prefix'], 0, 3);
			}

			if (in_array($_REQUEST['payos_gateway_settings']['transaction_prefix'], self::$BLACK_LIST_PREFIX)) {
				$_REQUEST['payos_gateway_settings']['transaction_prefix'] = self::$default_settings["transaction_prefix"];
			}
			update_option('woocommerce_payos_gateway_settings', $_REQUEST['payos_gateway_settings']);
		}
	}

	/**
	 * Initialise Gateway Settings Form Fields. 
	 * 
	 */
	public function console_log($output, $with_script_tags = true)
	{
		$js_code = 'console.log(' . wp_json_encode($output, JSON_HEX_TAG) .
			');';
		if ($with_script_tags) {
			$js_code = '<script>' . $js_code . '</script>';
		}
		// phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
		echo $js_code;
	}
	public function init_form_fields()
	{
		$this->form_fields = array(
			'enabled'         => array(
				'title'   => __('Enable/Disable', 'woocommerce'),
				'type'    => 'checkbox',
				'label'   => __('Enable bank transfer', 'woocommerce'),
				'default' => 'true',
			),
			'title'           => array(
				'title'       => __('Title', 'woocommerce'),
				'type'        => 'text',
				'description' => __('This controls the title which the user sees during checkout.', 'woocommerce'),
				'default'     => __('Payment by bank transfer (Scan VietQR)', 'casso-payos'),
				'desc_tip'    => true,
			),
			'description'     => array(
				'title'       => __('Description', 'woocommerce'),
				'type'        => 'textarea',
				'description' => __('Payment method description that the customer will see on your checkout.', 'woocommerce'),
				'default'     => __('Pay for orders via payOS. Supported by almost Vietnamese banking apps', 'casso-payos'),
				'desc_tip'    => true,
			),
			'payos_gateway_settings' => array(
				'type' => 'payos_gateway_settings',
			)
		);
	}

	/**
	 * Generate account details html.
	 *
	 * @return string
	 */
	public function generate_payos_gateway_settings_html()
	{
		ob_start();
		$settings = self::get_payos_gateway_settings();

		// wp_enqueue_script('jQuery');
		wp_enqueue_script('payos-select2-script', plugins_url('../assets/js/select2.min.js', __FILE__), array(), '1.0.0', true);
		wp_enqueue_style('payos-select2-styles', plugins_url('../assets/css/select2.min.css', __FILE__), array(), '1.0.0', true);
		$queries = array();
		parse_str($_SERVER['QUERY_STRING'], $queries);
		$is_claim_show = false;
		if (isset($queries['claim'])) $is_claim_show = true;
?>
		<script>
			jQuery(document).ready(function() {
				jQuery("#payos_bank_selector2").select2({
					templateResult: formatOptions
				});
			});

			function formatOptions(state) {
				if (!state.id) {
					return state.text;
				}
				var $state = jQuery(
					'<span ><img style="vertical-align: middle;" src="' + state.title + '"  width="80px"/> ' + state.text + '</span>'
				);
				return $state;
			}
		</script>
		<?php echo wp_kses_post($this->message) ?>
		<input type="hidden" id="action" name="action" value="payos_save_settings">
		<input type="hidden" id="payos_nonce" name="payos_nonce" value="<?php echo esc_attr(wp_create_nonce('payos_save_settings')) ?>">
		<?php if ($this->payos_gateway_settings['use_payment_gateway'] == 'yes') {
			$payment_gateway_config =
				'
		<tr>
			<th scope="row">' . __('Connection Information', 'casso-payos') . '</th>
			<td class="forminp" id="payos_gateway_settings">
				<button id="payos_info_button" onclick="showDetailGateway()" type="button" style="border: none; cursor:pointer;';

			if ($this->payos_gateway_settings['link_webhook'] == 'no')
				$payment_gateway_config .= 'background-color: #af1818;';
			else
				$payment_gateway_config .= 'background-color: #15ab64;';

			$payment_gateway_config .=	'color: white; border-radius: 5px; padding: 6px; min-width: 125px !important; margin-bottom: 10px;">';

			if ($this->payos_gateway_settings['link_webhook'] == 'no') {
				$payment_gateway_config .= __('No Connection', 'casso-payos') . '</button>';
			} else {
				$payment_gateway_config .=  __('Connected', 'casso-payos') . '</button>
					<div id="payos_gateway_info" style="padding: 20px; background-color: #bdbdbd; color: black; border: 1px solid; max-width: 500px; border-radius: 10px; display: none">
						<ul';
				if ($this->payos_gateway_settings['link_webhook'] == 'no') $payment_gateway_config .= ' style="display: none"';
				$payment_gateway_config .= '>
							<li><b>' . __('Gateway name', 'casso-payos') . ': </b>' . esc_html($this->payos_gateway_settings['gateway_info']['name']) . '</li>
							<li><b>' . __('Account number', 'casso-payos') . ': </b>' . esc_html($this->payos_gateway_settings['gateway_info']['account_number']) . '</li>
							<li><b>' . __('Account name', 'casso-payos') . ': </b>' . esc_html($this->payos_gateway_settings['gateway_info']['account_name']) . '</li>	
							<li><b>' . __('Bank', 'casso-payos') . ': </b>' . esc_html($this->payos_gateway_settings['gateway_info']['bank_name']) . '</li>	
						</ul>
					</div>';
			}

			$payment_gateway_config .= '
				<input type="text" name="payos_gateway_settings[link_webhook]" value="' . esc_html($this->payos_gateway_settings['link_webhook']) . '" style="display: none" />
				</br>
				<button id="toggle_payos_gateway_settings" onClick="togglePayOSSetting()" style="border: none; text-decoration: underline; color: #2271b1">' . __('Enter information of payOS', 'casso-payos') . '</button>
			</td>
		</tr>
		<tbody id="payos_gateway_settings_group" style="display:none">
			<tr valign="top">
				<th scope="row" class="titledesc">' . esc_html(__('Client ID', 'casso-payos')) . ':</th>
				<td class="forminp" id="payos_gateway_settings">
					<input type="password" value="' . esc_html($this->payos_gateway_settings['client_id']) . '" name="payos_gateway_settings[client_id]" id="payos_client_id"/>
					<input id="show_client_id" onclick="showClientId()" type="button" style="border: none; background-color: #f0f0f1; min-width: 45px !important; color: #056fca; border-radius: 5px; padding: 6px;" value="' . __('Show', 'casso-payos') . '" /> 
				</td>
			</tr>
			<tr valign="top">
				<th scope="row" class="titledesc">' . esc_html(__('API Key', 'casso-payos')) . ':</th>
				<td class="forminp" id="payos_gateway_settings">
					<input type="password" value="' . esc_html($this->payos_gateway_settings['api_key']) . '" name="payos_gateway_settings[api_key]" id="payos_api_key"/>
					<input id="show_api_key" onclick="showApiKey()" type="button" style="border: none; background-color: #f0f0f1; min-width: 45px !important; color: #056fca; border-radius: 5px; padding: 6px;" value="' . __('Show', 'casso-payos') . '" /> 
				</td>
			</tr>
			<tr valign="top">
				<th scope="row" class="titledesc">' . esc_html(__('Checksum Key', 'casso-payos')) . ':</th>
				<td class="forminp" id="payos_gateway_settings">
					<input type="password" value="' . esc_html($this->payos_gateway_settings['checksum_key']) . '" name="payos_gateway_settings[checksum_key]" id="payos_checksum_key"/>
					<input id="show_checksum_key" onclick="showChecksumKey()" type="button" style="border: none; background-color: #f0f0f1; min-width: 45px !important; color: #056fca; border-radius: 5px; padding: 6px;" value="' . __('Show', 'casso-payos') . '" /> 
					<br>
					<div style="margin-top: 8px">
						<input type="submit" name="submit" id="submit" class="button button-primary" value="' . __('Check connection payOS', 'casso-payos') . '" style="border: none; background-color: #056fca; color: white; border-radius: 5px; min-width: 95px !important;">
						<a href="https://payos.vn/docs/huong-dan-su-dung/tao-kenh-thanh-toan/" onclick="window.open(this.href); return false;" onkeypress="window.open(this.href); return false;" style="margin-left: 5px; border: none; color: #2271b1; text-decoration: underline; line-height: 30px">' . __('Instructions to get Client ID and API Key', 'casso-payos') . '</a>
					</div>
					';
			$payment_gateway_config .= '
				</td>
			</tr>
		</tbody>
		<tr>
			<th scope="row">' . esc_html(__('Prefix:', 'casso-payos')) . '</th>
			<td id="payos_gateway_settings">
				<input name="payos_gateway_settings[transaction_prefix]" type="text" value="' . esc_html($this->payos_gateway_settings['transaction_prefix']) . '" id="transaction_prefix"> 
				<label for="transaction_prefix" style="font-size: 13px; font-style: oblique;">' . esc_html(__('Maximum 3 characters, no spaces and no special characters. If contained, it will be deleted. Please do not prefix starting with FT, TF, TT, VQR', 'casso-payos')) . '</label>
			</td>
		</tr>
		<tr>
			<th scope="row">' . esc_html(__('Status if payment is successful:', 'casso-payos')) . '</th>
			<td id="payos_gateway_settings">
				<select name="payos_gateway_settings[order_status][order_status_after_paid]" id="order_status_after_paid">';
			foreach ($this->casso_get_order_statuses_after_paid() as $key => $value) {
				if ($key == $this->payos_gateway_settings["order_status"]["order_status_after_paid"])
					$payment_gateway_config .= '<option value="' . esc_attr($key) . '" selected>' . esc_attr($value) . '</option>';
				else
					$payment_gateway_config .= '<option value="' . esc_attr($key) . '" >' . esc_attr($value) . '</option>';
			}
			$payment_gateway_config .= '</select>
			</td>
		</tr>
		<tr valign="top">
			<th scope="row">' . esc_html(__('Status if payment is failed:', 'casso-payos')) . '</th>
			<td  id="payos_gateway_settings">
				<select name="payos_gateway_settings[order_status][order_status_after_underpaid]" id="order_status_after_underpaid">';
			foreach ($this->casso_get_order_statuses_after_underpaid() as $key => $value) {
				if ($key == $this->payos_gateway_settings['order_status']['order_status_after_underpaid'])
					$payment_gateway_config .= '<option value="' . esc_attr($key) . '" selected>' . esc_attr($value) . '</option>';
				else
					$payment_gateway_config .= '<option value="' . esc_attr($key) . '">' . esc_attr($value) . '</option>';
			}
			$payment_gateway_config .= '</select>
			</td>
		</tr>
		<tr valign="top" style="display: none">
			<th scope="row">' . esc_html(__('Redirect payOS:', 'casso-payos')) . '</th>
			<td  id="payos_gateway_settings">
				<input name="payos_gateway_settings[redirect]" type="hidden" value="no">
				<input name="payos_gateway_settings[redirect]" type="checkbox" value="yes" id="redirect" ';
			if (!empty($this->payos_gateway_settings['redirect']) && $this->payos_gateway_settings['redirect'] == 'yes')
				$payment_gateway_config .= 'checked="checked"';

			$payment_gateway_config .= '/>
				<label for="payos_gateway_settings[redirect]">' . esc_html(__('On/Off', 'casso-payos')) . '</label>
			</td>
		</tr>
		
		<script type="text/javascript">
			function showDetailGateway() {
				var dots = document.getElementById("payos_gateway_info");
				var button = document.getElementById("payos_info_button");

				if(!dots) return;
				if (dots.style.display === "none") {
					dots.style.display = "block";
					button.innerHTML = "' . __('Show less', 'casso-payos') . '";
				} else {
					dots.style.display = "none";
					button.innerHTML = "';
			if ($this->payos_gateway_settings['link_webhook'] == 'no')
				$payment_gateway_config .= __('No Connection', 'casso-payos');
			else
				$payment_gateway_config .= __('Connected', 'casso-payos');
			$payment_gateway_config .=	'";
				}
			}

			function showClientId() {
				var clientId = document.getElementById("payos_client_id");
				var showClientIdBtn = document.getElementById("show_client_id");
				if (clientId.type === "password") {
					clientId.type = "text";
					showClientIdBtn.value = "' . __('Hide', 'casso-payos') . '";
				} else {
					clientId.type = "password";
					showClientIdBtn.value = "' . __('Show', 'casso-payos') . '";
				}
			}

			function showApiKey() {
				var apiKey = document.getElementById("payos_api_key");
				var showApiKeyBtn = document.getElementById("show_api_key");
				if (apiKey.type === "password") {
					apiKey.type = "text";
					showApiKeyBtn.value = "' . __('Hide', 'casso-payos') . '";
				} else {
					apiKey.type = "password";
					showApiKeyBtn.value = "' . __('Show', 'casso-payos') . '";
				}
			}

			function showChecksumKey() {
				var checksumKey = document.getElementById("payos_checksum_key");
				var showChecksumKeyBtn = document.getElementById("show_checksum_key");
				if (checksumKey.type === "password") {
					checksumKey.type = "text";
					showChecksumKeyBtn.value = "' . __('Hide', 'casso-payos') . '";
				} else {
					checksumKey.type = "password";
					showChecksumKeyBtn.value = "' . __('Show', 'casso-payos') . '";
				}
			}

			function togglePayOSSetting() {
				event.preventDefault();
				var toggleBtn = document.getElementById("toggle_payos_gateway_settings");
				var payOSGatewayGroup = document.getElementById("payos_gateway_settings_group");
				if (payOSGatewayGroup.style.display == "none") {
					payOSGatewayGroup.style.display = "table-row-group";
				}
				else {
					payOSGatewayGroup.style.display = "none";
				}
			}
		</script>
		';
			// phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
			echo $payment_gateway_config;
		}
		?>
<?php
		return ob_get_clean();
	}

	/**
	 * Output for the order received page.
	 *
	 * @param int $order_id Order ID.
	 */
	public function thankyou_page($order_id)
	{
		echo  '
		<script>
			const isMobileScreen = () => Boolean(
				navigator.userAgent.match(/Android/i) ||
				navigator.userAgent.match(/webOS/i) ||
				navigator.userAgent.match(/iPhone/i) ||
				navigator.userAgent.match(/iPad/i) ||
				navigator.userAgent.match(/iPod/i) ||
				navigator.userAgent.match(/BlackBerry/i) ||
				navigator.userAgent.match(/Windows Phone/i)
			);
			const payOSContent = document.currentScript.parentElement;
			const content = document.createElement("div");
			content.style.height = isMobileScreen() ? "620px" : "340px";
			content.style.width = "100%";
			payOSContent.appendChild(content);
			const beforeUnloadHandler = (e) => {
			 	e.preventDefault;
				e.returnValue = true;
			};
			window.addEventListener("beforeunload", beforeUnloadHandler);
			setTimeout(() => {
				content.scrollIntoView({behavior: "smooth"});
			}, 500);
		</script>';
		// loading ui
		$this->use_payment_gateway_template();
		$logger = wc_get_logger();
		try {
			$order = wc_get_order($order_id);
			if (!$order) {
				throw new Exception(sprintf('Not found order #%d', $order_id));
			}
			// order not use payOS, do nothing
			if ($this->id !== $order->get_payment_method()) {
				return;
			}

			// return if paid
			if (str_contains($this->payos_gateway_settings["order_status"]["order_status_after_paid"], $order->get_status())) {
				$this->use_payment_gateway_template('PAID');
				return;
			}
		} catch (Exception $e) {
			// if there was an error, log to a file and add an order note
			$logger->error(
				sprintf('Error when get order #%d', $order_id),
				array('order' => $order_id, 'error' => $e->getMessage())
			);
			$this->use_payment_gateway_template('ERROR');
			return;
		}
		// get order from payOS
		try {
			$response = $this->handle_fetch_payos(self::$URL_GET_PAYMENT_LINK_INFO . $order_id, 'GET');

			// check status if pending return checkout_url
			if ($response['status'] === 'PENDING') {
				$checkout_url = self::$PAYOS_CHECKOUT_HOST . '/web/' . $response['id'];
				$this->use_payment_gateway_template('PENDING', $checkout_url, $order);
				return;
			}

			// check amount
			if ((string) $response['amount'] !== $order->get_total()) {
				throw new Exception(sprintf('Can not use this payment gateway on this store'));
			}

			if ($response['status'] === 'PAID') {
				$this->use_payment_gateway_template('PAID');
				return;
			}
		} catch (Exception $e) {
			if ($e->getCode() !== self::$PAYOS_NOT_FOUND_ORDER_CODE) {
				$logger->error(sprintf('Error when get PayOS payment link #%d', $order_id), array('order' => $order_id, 'error' => $e->getMessage()));
				$this->use_payment_gateway_template('ERROR');
				return;
			}
		}

		// create payment link
		try {
			$checkout_url = $this->get_payos_payment_url($order_id);
			$this->use_payment_gateway_template('PENDING', $checkout_url, $order);
		} catch (Exception $e) {
			$logger->error('Error when create payment link', array('order' => $order_id, 'error' => $e->getMessage()));
			$this->use_payment_gateway_template('ERROR');
			return;
		}
	}

	/**
	 * Add content to the WC emails.
	 *
	 * @param WC_Order $order Order object.
	 * @param bool     $sent_to_admin Sent to admin.
	 * @param bool     $plain_text Email format: plain text or HTML.
	 */
	public function email_instructions($order, $sent_to_admin, $plain_text = false)
	{
		if (!$sent_to_admin && $this->id === $order->get_payment_method() && $order->has_status('on-hold')) {
			if ($this->instructions) {
				echo wp_kses_post(wpautop(wptexturize($this->instructions)) . PHP_EOL);
			}
		}
	}

	/**
	 * Process the payment and return the result.
	 *
	 * @param int $order_id Order ID.
	 * @return array
	 */
	public function process_payment($order_id)
	{
		$order = wc_get_order($order_id);
		if ($order->get_total() > 0) {
			// Mark as on-hold (we're awaiting the payment).
			$order->update_status(apply_filters('woocommerce_payos_process_payment_order_status', 'on-hold', $order), __('Awaiting payment', 'woocommerce'));
		} else {
			$order->payment_complete();
		}

		// Remove cart.
		WC()->cart->empty_cart();

		// Return thankyou redirect.
		return array(
			'result'   => 'success',
			'redirect' => $this->get_return_url($order),
		);
	}

	public function get_payos_payment_url($order_id)
	{
		$order = wc_get_order($order_id);
		$payos_gateway_settings = self::get_payos_gateway_settings();
		if ($order->get_payment_method() !== 'casso-payos') {
			throw new Exception('Not use PayOS as payment method');
		}
		if (!isset($payos_gateway_settings['use_payment_gateway']) || $payos_gateway_settings['use_payment_gateway'] !== 'yes') {
			throw new Exception('Not use PayOS payment gateway');
		}
		try {
			parse_str($_SERVER['QUERY_STRING'], $query_str_arr);
			if ($query_str_arr && array_key_exists("code", $query_str_arr)) return;

			$order_id = $order->get_id();
			$woo_checkout_url = wc_get_checkout_url();
			if (substr($woo_checkout_url, -1) != '/') {
				$woo_checkout_url = $woo_checkout_url . '/';
			}
			$redirect_url = $order->get_checkout_order_received_url();

			$item_cart = array();
			foreach ($order->get_items() as $item_id => $item) {
				$product_name = $item->get_name();
				$quantity = $item->get_quantity();
				$total = intval($item->get_total());
				array_push($item_cart, array(
					'name' => $product_name,
					'quantity' => $quantity,
					'price' => $total
				));
			}
			$data = array(
				"orderCode" => $order->get_id(),
				"description" => $this->payos_gateway_settings['transaction_prefix'] . $order->get_id(),
				"amount" => intval($order->get_total()),
				"buyerName" => $order->get_billing_first_name() . ' ' . $order->get_billing_last_name(),
				"buyerEmail" => $order->get_billing_email(),
				// "taxCode" => $order->get_meta("tax_code"),
				"items" => $item_cart,
				"returnUrl" => $redirect_url,
				"cancelUrl" => $redirect_url
			);
			// Tao signature cho data request
			$request_data_signature = $this->create_signature_payment_request($data["amount"], $data["cancelUrl"], $data["description"], $data["orderCode"], $data["returnUrl"]);
			$data["signature"] = $request_data_signature;

			$result = $this->handle_fetch_payos(self::$URL_CREATE_PAYMENT_LINK, 'POST', $data);
			return $result['checkoutUrl'];
		} catch (Exception $e) {
			throw new Exception(esc_html($e->getMessage()), intval($e->getCode()));
		}
	}

	public function order_received_text($text, $order)
	{
		if ($order && $order->get_payment_method() === 'casso-payos') {
			$status = $order->get_status();
			switch ($status) {
				case str_contains($this->payos_gateway_settings["order_status"]["order_status_underpaid"], $order->get_status()):
					return esc_html__('Your order has been cancelled.', 'casso-payos');
				case str_contains($this->payos_gateway_settings["order_status"]["order_status_after_paid"], $order->get_status()):
					return esc_html__('Thank you. Your order has been fulfilled.', 'casso-payos');
				default:
					return esc_html__('Thank you. Your order is processing.', 'casso-payos');
			}
		}
		return $text;
	}

	public function use_payment_gateway_template(string $status = null, string $checkout_url = null, $order = null)
	{
		if (empty($this->account_details) && isset($this->account_details)) {
			return;
		}
		$payment_html = '';

		switch ($status) {
			case 'PENDING':
				$order_id = $order->get_id();
				$order = wc_get_order($order_id);
				$redirect_url = $order->get_checkout_order_received_url();
				$order = wc_get_order($order_id);
				$checkout_url = str_replace('/web/', '/embedded/', $checkout_url);
				$payment_html .= '
<script>
	let paymentLinkOrigin = null;
	const isJsonString = (str) => {
		if (!str) {
			return false;
		}
		try {
			JSON.parse(str);
			return true;
		} catch (e) {
			return false;
		}
	};
	const redirectUrl = `' . $redirect_url . '`;
	const checkoutUrl = `' . $checkout_url . '`;
	const paymentLinkDialogUrl = checkoutUrl + "?redirect_uri=" + encodeURIComponent(redirectUrl);
	try {
		const handlePostMessage = async (event) => {
			if (event.origin !== new URL(paymentLinkOrigin).origin) {
				return;	
			}
			const eventData = isJsonString(event.data) ? JSON.parse(event.data) : undefined;
			if (!eventData) {
				return;
			}							
			if (eventData.type !== "payment_response") return;
			const responseData = eventData.data;
			if (responseData?.status === "PAID") {
				window.removeEventListener("beforeunload", beforeUnloadHandler);
				content.innerHTML = `' . $this->show_ui(__('Order has been successfully paid.', 'casso-payos'), 'success.png', false) . '`;
			}	
		};
		paymentLinkOrigin = `' . $checkout_url . '`;
		content.innerHTML =`<iframe src=${paymentLinkDialogUrl} style="height: 100%; width: 100%; border: none" allow="clipboard-read; clipboard-write"></iframe>`;
		window.addEventListener("message", handlePostMessage);
	} catch (error) {
		content.innerHTML = `' . $this->show_ui(__('Cannot show payment link', 'casso-payos'), 'failed.png', true) . '`;
		window.removeEventListener("beforeunload", beforeUnloadHandler);
	}
</script>
				';
				break;
			case 'PAID':
				$payment_html .= '<script>content.innerHTML = `' . $this->show_ui(__('Order has been successfully paid.', 'casso-payos'), 'success.png', false) . '`; window.removeEventListener("beforeunload", beforeUnloadHandler);</script>';
				break;
			case 'ERROR':
				$payment_html .= '<script>content.innerHTML = `' . $this->show_ui(__('Cannot show payment link', 'casso-payos'), 'failed.png', true) . '`; window.removeEventListener("beforeunload", beforeUnloadHandler);</script>';
				break;
			default:
				$payment_html .= '<script>
				content.innerHTML=`
					 <style>
					.loader-payos {
						border: 10px solid #f3f3f3; 
						border-top: 10px solid #6655ff; 
						border-radius: 50% !important;
						width: 50px;
						height: 50px;
						animation: spin-payos 2s linear infinite;
					}

					@keyframes spin-payos {
						0% { transform: rotate(0deg); }
						100% { transform: rotate(360deg); }
					}
					</style>
					<div style="display:flex; flex-direction: column; justify-content: center; align-items: center; height: 100%">
						<div style="display: flex; flex-direction: column; gap: 10px; text-align: center; padding: 20px; align-items: center">
						<div class="loader-payos"></div>
						<p>Vui lòng chờ ...</p>
						</div>
					</div>`;
				</script>
				';
		}

		// phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped	
		echo $payment_html;
	}

	public function show_ui(string $message, string $img_file_name, bool $is_error)
	{
		$color = $is_error ? "#D32F2F" : "#6655ff";
		return '
<div style=\'display:flex; flex-direction: column; justify-content: center; align-items: center; height: 100%\'>
	<div style=\'text-align: center; padding: 20px;\'>
		<img src=\'' . plugins_url('../assets/img/' . $img_file_name, __FILE__) . '\' style=\'width: 50px; height: 50px; margin-bottom: 10px\'/>
		<p style=\'color:' . $color . '; font-size:20px\'>' . $message . '</p>
	</div>
</div>';
	}

	public
	function casso_get_order_statuses_after_paid()
	{
		$wooRemovedStatuses = array(
			'wc-pending',
			// 'wc-processing',
			// 'wc-on-hold',
			'wc-completed',
			'wc-cancelled',
			'wc-refunded',
			'wc-failed',
		);
		$statuses =  wc_get_order_statuses();
		for ($i = 0; $i < count($wooRemovedStatuses); $i++) {
			$statusName = $wooRemovedStatuses[$i];
			if (isset($statuses[$statusName])) {
				unset($statuses[$statusName]);
			}
		}
		return $statuses;
	}

	public function casso_get_order_statuses_after_underpaid()
	{
		$wooRemovedStatuses = array(
			'wc-pending',
			'wc-processing',
			'wc-on-hold',
			'wc-completed',
			'wc-cancelled',
			'wc-refunded',
			// 'wc-failed',
		);
		$statuses =  wc_get_order_statuses();
		for ($i = 0; $i < count($wooRemovedStatuses); $i++) {
			$statusName = $wooRemovedStatuses[$i];
			if (isset($statuses[$statusName])) {
				unset($statuses[$statusName]);
			}
		}
		return $statuses;
	}

	public function verify_payment_handler()
	{
		$logger = wc_get_logger();
		$txtBody = file_get_contents('php://input');
		$body = json_decode(str_replace('\\', '\\\\', $txtBody), true);
		try {
			if ($body['desc'] == 'Giao dich thu nghiem' || $body['data']['description'] == 'VQRIO123' || $body['data']['reference'] == 'MA_GIAO_DICH_THU_NGHIEM') {
				echo 'Webhook delivered successfully';
				die();
			}
			if ($body['code'] !== self::$PAYOS_SUCCESS) {
				throw new Exception(strval($body['desc']), intval($body['code']));
			}
			$transaction = $body['data'];
			// --------------Xac thuc du lieu webhook-------------------
			$signature = $this->create_signature($transaction);
			if (!$body['signature'] || $signature !== $body['signature']) {
				throw new Exception('Data not integrity');
			}
			// ---------------------------------------------------------
			// Xu ly du lieu giao dich
			$order_code = $transaction['orderCode'];
			$order = wc_get_order($order_code);
			$payos_gateway_settings = get_option('woocommerce_payos_gateway_settings');
			if (!$order) {
				throw new Exception(sprintf('Error when get order #%d', $order_code));
			}
			$rest_amount = intval($transaction['amount']) - intval($order->get_total());
			switch (true) {
				case $rest_amount >= 0:
					$order->payment_complete();
					wc_reduce_stock_levels($order_code);
					if (isset($payos_gateway_settings['order_status']['order_status_after_paid'])) {
						$order->update_status($payos_gateway_settings['order_status']['order_status_after_paid']);
					}
					if ($rest_amount > 0) {
						$order->add_order_note(__('Order has been overpaid', 'casso-payos'));
					}

					break;
				case $rest_amount < 0:
					if (isset($payos_gateway_settings['order_status']['order_status_after_underpaid'])) {
						$order->update_status($payos_gateway_settings['order_status']['order_status_after_underpaid']);
						$order->add_order_note(__('Order has been underpaid', 'casso-payos'));
					}
					break;
				default:
					break;
			}
			$transaction_note = sprintf(
				/* translators: 1: order code, 2: amount, 3: description, 4: account number, 5: reference */
				__('<b>Transaction Information:</b> <br> Order Code: %1$s <br> Amount: %2$s <br> Description: %3$s <br> Account Number: %4$s <br> Reference: %5$s', 'casso-payos'),
				$transaction['orderCode'],
				$transaction['amount'],
				$transaction['description'],
				$transaction['accountNumber'],
				$transaction['reference']
			);
			$order->add_order_note($transaction_note);
			echo 'Webhook delivered successfully';
			die();
		} catch (Exception $e) {
			$logger->error(wc_print_r($body, true), array('error' => $e->getMessage(), 'source' => 'webhooks-delivery'));
			$response = array('code' => $e->getCode(), 'message' => $e->getMessage());
			echo wp_json_encode($response);
			die();
		}
	}

	public function payos_payment_page($order_id)
	{
		$order = wc_get_order($order_id);
		if ($this->id === $order->get_payment_method()) {
			$payment_url = $this->get_payos_payment_url($order_id);
			header("Location: {$payment_url}");
			die();
		}
	}

	public function payos_create_webhook($settings)
	{
		$response = null;
		$body = array(
			"webhookUrl" =>  self::get_webhook_url()
		);

		$url  = self::$URL_CONFIRM_WEBHOOK;
		$args = array(
			'body'        => wp_json_encode($body),
			'headers' => array(
				"Content-Type" 	=> "application/json",
				"x-client-id" 	=> $settings['client_id'],
				"x-api-key" 	=> $settings['api_key']
			)
		);
		$response = wp_remote_post($url, $args);

		if (is_wp_error($response)) {
			return null;
		}
		$body_response = json_decode($response['body'], true);
		if (($response['response']['code'] == 200 || $response['response']['code'] == 201) && $body_response['code'] == '00') {
			$body     = wp_remote_retrieve_body($response);
			return $body;
		}
		return null;
	}

	public function create_signature($data)
	{
		ksort($data);
		$data_str_arr = [];
		foreach ($data as $key => $value) {
			if ($value === "undefined" || $value === "null" || gettype($value) == "NULL") {
				$value = "";
			}
			if (is_array($value)) {
				$value_sorted_ele_obj = array_map(function ($ele) {
					ksort($ele);
					return $ele;
				}, $value);
				$value = wp_json_encode($value_sorted_ele_obj, JSON_UNESCAPED_UNICODE);
			}
			$data_str_arr[] = $key . "=" . $value;
		}
		$data_str = implode('&', $data_str_arr);
		$signature = hash_hmac('sha256', $data_str, $this->payos_gateway_settings['checksum_key']);
		return $signature;
	}

	public function create_signature_payment_request($amount, $cancel_url, $description, $order_code, $return_url)
	{
		$data_str = "amount={$amount}&cancelUrl={$cancel_url}&description={$description}&orderCode={$order_code}&returnUrl={$return_url}";
		$signature = hash_hmac('sha256', $data_str, $this->payos_gateway_settings['checksum_key']);
		return $signature;
	}

	static function get_webhook_url()
	{
		return WC()->api_request_url(self::$WEBHOOK_ROUTE);
	}

	public function handle_fetch_payos(string $url, string $method = 'GET', $data = null)
	{
		$client_id = $this->payos_gateway_settings['client_id'];
		$api_key = $this->payos_gateway_settings['api_key'];
		$args = [
			'method'    => $method,
			'headers'   => [
				'Content-Type' => 'application/json',
				'x-client-id'  => $client_id,
				'x-api-key'    => $api_key,
			],
			'body'      => $data ? wp_json_encode($data) : '',
			'timeout'   => 45,
		];

		$response = wp_remote_request($url, $args);

		if (is_wp_error($response)) {
			throw new Exception(esc_html($response->get_error_message()));
		}

		$body = wp_remote_retrieve_body($response);
		$result = json_decode($body, true);

		if ($result['code'] !== self::$PAYOS_SUCCESS) {
			throw new Exception(esc_html($result['desc']), intval($result['code']));
		}

		// Check data integrity by verifying the signature
		$checkout_response = $result['data'];
		$checkout_response_signature = $this->create_signature($checkout_response);

		if ($checkout_response_signature !== $result['signature']) { // Data integrity check failed
			throw new Exception('Data not integrity');
		}

		return $checkout_response;
	}
}
